//+------------------------------------------------------------------+
//|                                                  AlexanderEA.mq5 |
//+------------------------------------------------------------------+

#property version   "1.0"

#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>  
#include <Trade\AccountInfo.mqh>
#include <Trade\OrderInfo.mqh>
              
CPositionInfo  m_position;                   // trade position object
CTrade         m_trade;                      // trading object
CSymbolInfo    m_symbol;                     // symbol info object
CAccountInfo   m_account;                    // account info wrapper
COrderInfo     m_order;                      // pending orders object


// Globale Eingabeparameter
input double Lot              = 0.01;     // Lot-Größe
input int minTick             = 10;    // Mindest-Tick-Differenz
input int TP                  = 80;    // Take Profit (in Punkten)
input int SL                  = 20;      // Stop Loss (in Punkten)
input int TS                  = 5;       // Trailing Stop (in Punkten)
input uchar InpMaxPositions   = 1;       // Maximum positions ("1" - you can run on netting accounts)
input int MagicNr             = 123456;  // Magic-Nummer

// Globale Variablen
ulong ticketNr;
double lastBid;

//+------------------------------------------------------------------+
//| OnInit()
//+------------------------------------------------------------------+
int OnInit()
{
   Comment("START");
   
   lastBid = m_symbol.Bid();
   m_symbol.Name(Symbol());  // set symbol name
   m_trade.SetExpertMagicNumber(MagicNr);
   
   return(INIT_SUCCEEDED);
}


//+------------------------------------------------------------------+
//| OnTick()
//+------------------------------------------------------------------+
void OnTick()

{   
   // Position vorhanden?
   bool hasPosition = false;
   for (int i = PositionsTotal() - 1; i >= 0; i--) {
      ticketNr = PositionGetTicket(i); // Position laden
      
      if (m_position.SelectByTicket(ticketNr)) {  // Position laden
         if (m_position.Symbol() != m_symbol.Name()) continue; // anderes Instrument?
         if (m_position.Magic() != MagicNr) continue; // anderer EA?
         hasPosition = true;
         break;
      }
   }
   
   // Trailing Stop
   if (hasPosition) {
      trailPositions();
      lastBid = m_symbol.Bid(); // letzten Preis merken
      return; // nicht mehr kaufen
   }

   // Kursdaten aktualisieren
   if (!m_symbol.RefreshRates()) return;

   // Kaufen/Verkaufen
   if (minTick <= 0) return; // Ausschalter
   
   if (lastBid != m_symbol.Bid()) {
       if (m_symbol.Bid() >= (lastBid + minTick)) {
           double normalizedSL = SL > 0 ? NormalizeDouble(m_symbol.Ask() + SL * _Point, _Digits) : 0;
           double normalizedTP = TP > 0 ? NormalizeDouble(m_symbol.Ask() + TP * _Point, _Digits) : 0;
           double normalizedLot = NormalizeLots(Lot);

           m_trade.Buy(normalizedLot, m_symbol.Name(), m_symbol.Ask(), normalizedSL, normalizedTP, "");
       } else if (m_symbol.Bid() <= (lastBid - minTick)) {
           double normalizedSL = SL > 0 ? NormalizeDouble(m_symbol.Bid() - SL * _Point, _Digits) : 0;
           double normalizedTP = TP > 0 ? NormalizeDouble(m_symbol.Bid() - TP * _Point, _Digits) : 0;
           double normalizedLot = NormalizeLots(Lot);

           m_trade.Sell(normalizedLot, m_symbol.Name(), m_symbol.Bid(), normalizedSL, normalizedTP, "");
       }
   }
   lastBid = m_symbol.Bid(); // letzten Preis merken
}

//+------------------------------------------------------------------+
// NormalizeLots
//+------------------------------------------------------------------+
double NormalizeLots(double value)
{
   double min = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN);
   double max = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MAX);
   double st = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_STEP);
   
   double rounded = st * (int)MathRound(value / st);
   rounded = MathMax(rounded, min);
   rounded = MathMin(rounded, max);
   rounded = NormalizeDouble(rounded, 2);
   return rounded;
}

//+------------------------------------------------------------------+
// trailPositions() 
//+------------------------------------------------------------------+
void trailPositions() 
{
   if (TS <= 0) return; // Prüfen, ob Trailing Stop deaktiviert ist
   
   double stopLoss = TS * _Point; // Trailing Stop in Punkten umrechnen
   
   for (int i = PositionsTotal() - 1; i >= 0; i--) {
      if (m_position.SelectByIndex(i) && m_position.Symbol() == Symbol() && m_position.Magic() == MagicNr) {
         ENUM_POSITION_TYPE type = m_position.PositionType();
         double CurrentSL = m_position.StopLoss();
         double CurrentTP = m_position.TakeProfit();
         double CurrentPrice = m_position.PriceCurrent();

         if (type == POSITION_TYPE_BUY) {
            if (CurrentPrice - stopLoss > CurrentSL || CurrentSL == 0.0) {
               m_trade.PositionModify(m_position.Ticket(), NormalizeDouble((CurrentPrice - stopLoss), _Digits), CurrentTP);
            }
         } else if (type == POSITION_TYPE_SELL) {
            if (CurrentPrice + stopLoss < CurrentSL || CurrentSL == 0.0) {
               m_trade.PositionModify(m_position.Ticket(), NormalizeDouble((CurrentPrice + stopLoss), _Digits), CurrentTP);
            }
         }
      }
   }
}

//+------------------------------------------------------------------+
// calculateProfit (History)
//+------------------------------------------------------------------+
int historyPositions = 0;
double historyProfit = 0.0;
   
void calculateProfit()
{
   historyPositions = 0;
   historyProfit = 0.0;
   
   HistorySelect(0, TimeCurrent());
   
   for (int i = (HistoryDealsTotal() - 1); i >= 0; i--) {
      ulong ticketDeal = HistoryDealGetTicket(i);
      
      if (ticketDeal > 0) {
         if (HistoryDealGetString(ticketDeal, DEAL_SYMBOL) != Symbol()) continue;
         if (HistoryDealGetInteger(ticketDeal, DEAL_MAGIC) != MagicNr) continue;
         
         if (HistoryDealGetInteger(ticketDeal, DEAL_ENTRY) == DEAL_ENTRY_OUT) {
            double profit = HistoryDealGetDouble(ticketDeal, DEAL_PROFIT)
                          + HistoryDealGetDouble(ticketDeal, DEAL_SWAP)
                          + HistoryDealGetDouble(ticketDeal, DEAL_COMMISSION);
            
            historyPositions++;
            historyProfit += profit;
         }
      }
   }
}
